/* Squint - Pentomino placement game
 * Copyright (C) 2005 David Turner &lt;novalis@novalis.org&gt;
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110
 */

#ifndef SQUINTCANVAS_H
#define SQUINTCANVAS_H

#include <qwidget.h>
#include <qcanvas.h>

#include <list>

#include "SquintBoard.h"
#include "SquintMove.h"
#include "SquintPieceItem.h"
#include "SquintPlayer.h"


const int SQUARE_WIDTH=17;

class SquintMove;

class SquintCanvas : public QCanvasView {

public:
  SquintCanvas (QWidget * parent = 0, const char * name = 0, Qt::WFlags f = 0);
  ~SquintCanvas();
  void cancelMove();
  virtual void contentsMousePressEvent(QMouseEvent *e);
  virtual void contentsMouseReleaseEvent(QMouseEvent *);
  virtual void contentsMouseMoveEvent(QMouseEvent *e);
  bool goodPlace(SquintPieceItem *piece, int gridx, int gridy);
  void buildGame();
  SquintPieceItem* findPieceByName(const QString& name);
  virtual void flip(int axis);
  virtual void rotate(int i = 0);

  void SquintCanvas::takeTurn(SquintMove* move);

  int board [9][12];
  SquintMove *current_move;

  SquintPlayer *turn;
  SquintPlayer *self;
  SquintPlayer *other;
protected:

  void SquintCanvas::placePiece(SquintPieceItem* piece, int gridx, int gridy);

  QCanvas* _canvas;

  SquintPieceItem* dragging_item;
  QPoint drag_start;

  SquintBoard* board_item;

  bool started;

};
#endif
