/* Squint - Pentomino placement game
 * Copyright (C) 2005 David Turner <novalis@novalis.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110
 */

#include "SquintPiece.h"

#include <algorithm>
using namespace std;

SquintPiece::SquintPiece (char* shape, int w, int h, const QString& n) :
  name (n), width (w), height(h), flip(0), rotation(0)
{

  for (int y = 0; y < 5; y++) {
    for (int x = 0; x < 5; x++) {
      grid[y][x] = 0;
    }
  }
  for (int y = 0; y < height; y++) {
    for (int x = 0; x < width; x++) {
      grid[y][x] = *(shape++) == '0' ? 0 : 1;
    }
  }
}

void SquintPiece::doHFlip() {
  for (int y = 0; y < height; y++) {
    for (int x = 0; x < width / 2; x++) {
      swap (grid[y][x], grid[y][width - x - 1]);
    }
  }

  //now, to handle the status bits
  flip = flip ^ 1;
  rotation = (4 - rotation) % 4;
}

void SquintPiece::doVFlip() {
  for (int y = 0; y < height / 2; y++) {
    for (int x = 0; x < width; x++) {
      swap (grid[y][x], grid[height - y - 1][x]);
    }
  }

  flip = flip ^ 1;
  if (!(rotation & 1))
    rotation ^= 2;
}

void SquintPiece::doRotate(int amount) {
  amount %= 4;
  if (amount == 0) return;
  if (amount & 1) {
    //a 90-degree rotation
    int newgrid[5][5];
    for (int y = 0; y < 5; y++) {
      for (int x = 0; x < 5; x++) {
	newgrid[y][x] = 0;
      }
    }
    for (int y = 0; y < height; y++) {
      for (int x = 0; x < width; x++) {
	if (amount == 1)
	  newgrid[x][height-y-1] = grid[y][x];
	else
	  newgrid[width-x-1][y] = grid[y][x];
      }
    }
    swap(width, height);
    for (int y = 0; y < height; y++) {
      for (int x = 0; x < width; x++) {
	grid[y][x] = newgrid[y][x];
      }
    }

  } else {
    //rotate in-place
    for (int y = 0; y < height / 2; y++) {
      for (int x = 0; x < width; x++) {
	swap(grid[y][x], grid[height - y - 1][width - x - 1]);
      }
    }

    //flip middle line, if any
    if (height % 2) {
      int y = (height / 2);
      for (int x = 0; x < width / 2; x++) {
	swap(grid[y][x], grid[y][width - x - 1]);

      }
    }
  }

  rotation += amount;
  rotation %= 4;

}


void SquintPiece::orient(int fl, int ro) {
  //first, re-orient to base
  doRotate(4 - rotation);

  //then, orient as specified
  if (flip != fl) doHFlip();
  doRotate (ro);

}
