/* Squint - Pentomino placement game
 * Copyright (C) 2005 David Turner <novalis@novalis.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110
 */

#include "SquintPieceItem.h"


SquintPieceItem::SquintPieceItem (QCanvas* parent, char* shape, int w, int h, const QString& n) :
  QCanvasRectangle(0, 0, 100, 100, parent),
  piece(shape, w, h, n)
{
  show();
}

void SquintPieceItem::doHFlip() {
  piece.doHFlip();
}

void SquintPieceItem::doVFlip() {
  piece.doVFlip();
}

void SquintPieceItem::doRotate(int arg) {
  piece.doRotate(arg);
}

QPoint SquintPieceItem::corner() {
  return QPoint (int(x()), int(y()));
}

void SquintPieceItem::snapToGrid(int gridx, int gridy) {
  move(gridx * SQUARE_WIDTH + 1, gridy * SQUARE_WIDTH + 1);
}

void SquintPieceItem::draw(QPainter &painter) {

  int xstart = x();
  int ystart = y();

  QPen qp (Qt::gray, 2, Qt::SolidLine);
  QBrush qb (Qt::gray);
  painter.setPen(qp);
  for (int y = 0; y < piece.height; ++y) {
    for (int x = 0; x < piece.width; ++x) {
      if (piece.grid[y][x]) 
	painter.fillRect(xstart + x * SQUARE_WIDTH, ystart + y * SQUARE_WIDTH,
			  SQUARE_WIDTH - 1, SQUARE_WIDTH - 1, qb);
    }
  }
}


int SquintPieceItem::rtti() const {
  return RTTI;
}

void SquintPieceItem::setHome(QPoint p) {
  home = p;
  goHome();
}

void SquintPieceItem::goHome() {
  move(home.x() + (5 - piece.width) * SQUARE_WIDTH / 2,
       home.y() + (5 - piece.height) * SQUARE_WIDTH / 2);
  update();
}
