/* Squint - Pentomino placement game
 * Copyright (C) 2005 David Turner <novalis@novalis.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110
 */

#include <qstringlist.h>

#include <cstdlib>

#include "SquintGame.h"


SquintGame::SquintGame (const QString& n, SquintClient *player) 
{
  for (int i = 0; i < 2; i++) {
    players[i] = 0;
  }
  restart(n, player);
}

void SquintGame::restart(const QString& n, SquintClient *player) {

  name = n;

  int good = 0;
  for (int i = 0; i < 2; i++) {
    if (players[i]) 
      good += 1;
  }

  if (good == 2) 
    startGame ();
  else {
    players[0] = player;
    players[1] = 0;
  }
}

void SquintGame::join(SquintClient* player) {
  players[1] = player;
  players[0]->announceJoin(players[1]);
  players[1]->announceYouJoin();
  players[1]->announceJoin(players[0]);
  startGame();
}

void SquintGame::startGame() {
  turn = random() % 2;

  for (int i = 0; i < 2; i++) {
    players[i]->startGame (turn == i);
  }
}

bool SquintGame::isTurn(SquintClient *client) {
  return players[turn] == client;
}

void SquintGame::move(const QString& move) {
  for (int i = 0; i < 2; ++i) {
    players[i]->sendMove (move);
  }
  turn ^= 1;
}


void SquintGame::clientDisconnected(SquintClient* client) {
  for (int i = 0; i < 2; ++i) {
    if (players [i] && players[i] != client) {
      players[i]->partnerDisconnected();
    }
  }
}

void SquintGame::chat(const QString& str, SquintClient* client) {
  for (int i = 0; i < 2; ++i) {
    if (players[i] != client) {
      QStringList args;
      args << QString("chat");
      args << str;
      players[i]->send(args);
    }
  }
}
