/* Squint - Pentomino placement game
 * Copyright (C) 2005 David Turner <novalis@novalis.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110
 */

#include "SquintServer.h"
#include <algorithm>
#include <functional>

using namespace std;

SquintServer::SquintServer(QObject *parent) :
  QServerSocket(10648,1,parent) 
{
  if (!ok()) {
    qWarning("Failed to bind to port 10648.");
    exit(1);
  }
}

void SquintServer::newConnection(int socketDescriptor) {

  QSocket* s = new QSocket( this );

  SquintClient* client = new SquintClient(s, this);
  clients.push_back(client);

  s->setSocket(socketDescriptor);

}

SquintGame* SquintServer::findGameByName(const QString& name) {

  for (GameList::iterator it = games.begin(); it != games.end(); ++it) {
    if ((*it)->name == name) {
      return *it;
    }
  }

  return 0;
}

void SquintServer::sendGames(SquintClient* client) {

  for (GameList::iterator it = games.begin(); it != games.end(); ++it) {
    client->announceGame(*it);
  }
}

void SquintServer::newGame(SquintGame* game) {
  games.push_back (game);
  ClientList::iterator client;
  for (ClientList::iterator it = clients.begin(); it != clients.end();) {
    client = it++;
    if ((*client)->dead) {
      clients.erase(client);
      continue;
    }
    (*client)->announceGame(game);
  }
}

bool SquintServer::removeGame(SquintGame* game) {
  GameList::iterator gameIt = find (games.begin(), games.end(), game);
  if (gameIt != games.end()) {
    //game may not be in game list (because it's already started)
    games.erase(gameIt);
    for (ClientList::iterator it = clients.begin(); it != clients.end(); ++it) {
      (*it)->unannounceGame(game);
    }
    return true;
  } else {
    return false;
  }
}

void SquintServer::discardClient(SquintClient* client) {

  if (client->game) {
    removeGame (client->game);
  }
}
